﻿/**
* Sample code for the GetArtifact Canada Post service.
* 
* The GetArtifact service is used to retrieve a pdf of the shipping label 
* created by a prior Create Non-Contract Shipment call. 
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.IO;
using System.Security.Authentication;
using System.Net;

namespace GetArtifact
{
    class GetArtifact
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username, password and customer number are imported from the following file
            // CPCWS_SOAP_NCShipping_DotNet_Samples\SOAP\returns\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];
            var mailedBy = ConfigurationSettings.AppSettings["customerNumber"];

            ArtifactPortTypeClient server = new ArtifactPortTypeClient("ArtifactPort", "https://ct.soa-gw.canadapost.ca/rs/soap/artifact");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            getartifactrequest request = new getartifactrequest();
            request.locale = "FR";
            request.artifactid = "10238";

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                getartifactresponse response = server.GetArtifact(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(getartifactresponseArtifactdata))
                {
                    getartifactresponseArtifactdata artifact = (getartifactresponseArtifactdata)response.Item;
                    var fileName = "artifact.pdf";
                    var encodedImage = artifact.image;
                    responseAsString += "Base64 Encoded: " + encodedImage + "\r\n";
                    responseAsString += "File name: " + fileName + "\r\n";
                    responseAsString += "Mime type: " + artifact.mimetype + "\r\n";
                    byte[] decodedImage = Convert.FromBase64String(encodedImage);
                    FileStream fileStream = File.Create(fileName);
                    fileStream.Write(decodedImage, 0, decodedImage.Length);
                    responseAsString += "Decoding to " + Directory.GetCurrentDirectory() + "\\" + fileName + "\r\n";
                    fileStream.Close();
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
